import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;

import javax.bluetooth.L2CAPConnection;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.List;
import javax.microedition.media.MediaException;
import javax.microedition.media.Player;
import javax.microedition.media.control.RecordControl;

class TMsg{
	int duration;
	byte[] msg;
}

public class Radio implements CommandListener {
	private String recString = "Rec";
	private Command cmdRec = new Command("Rec", Command.CANCEL, 2);	
	private Command cmdConsole = new Command("Console", Command.OK, 1);
	
	final int ST_IDLE = 0;
	final int ST_RECORDING = 1;
	final int ST_PLAYING = 2;
	
	private Vector messages = new Vector();
	private List soundsList = new List("null",List.IMPLICIT);
	DataInputStream iStream;
	DataOutputStream oStream;
	Display display;
	Displayable prevDisplay;
	private int isRecording = ST_IDLE;
	private boolean recOrSend = true;
	
    private Player player;
    private RecordControl rc;
	
	final int P_MESSAGE = 50000;
	
	public Radio(Display display){
		this.display = display;
		soundsList.addCommand(cmdRec);
		soundsList.addCommand(cmdConsole);
		soundsList.setCommandListener(this);
		new Thread(){
			public void run(){ processStream(); }
		}.start();
		listen();
	}
	
	public void show(){
		prevDisplay = display.getCurrent();
		display.setCurrent(soundsList);
	}
	
	private void onDisconnect(){
		BTConnection.onBTDisconnect();
		BTServer.onDisconnect();
		display.setCurrent(prevDisplay);
	}
	
	
	private void processStream(){
		while (true){
			if (BTConnection.conn != null){
				soundsList.setTitle(BTConnection.conn.toString());
				try{
					
					while (true){
						
						byte[] iArray = Misc.receiveArray(BTConnection.conn);
						ByteArrayInputStream biStream = new ByteArrayInputStream( iArray );
						
						iStream = new DataInputStream(biStream);
						int command = iStream.readInt();
						switch (command){
						case P_MESSAGE:{
							int length = iStream.readInt();
							int duration = iStream.readInt();
							TMsg msg = new TMsg();
							msg.msg = new byte[length];
							msg.duration = duration;
							for (int i = 0; i < length; i++){
								msg.msg[i] = iStream.readByte();
							}
							
							messages.addElement(msg);
							onMessages();
						}
						iStream.close();
						biStream.close();
						}
						try {Thread.sleep(50);} catch (InterruptedException e){};
					}
				}catch (Exception e){
					Console.trace("at procStream" + e.toString());
					onDisconnect();
				};
					
			}
			try {Thread.sleep(50);} catch (InterruptedException e){};
		}
			
	}
	
	private void onMessages(){
		soundsList.deleteAll();
		for (int i=0; i<messages.size();i++){
			soundsList.append( String.valueOf( ((TMsg)messages.elementAt(i)).duration/1000 ), null );
		}
		
	}
	
    private void informError(String msg){
        Alert al = new Alert("Error", msg, null, AlertType.ERROR);
        al.setTimeout(1000);
        display.setCurrent(al, display.getCurrent());
    }
	
    private void setRecording(){
    	isRecording = ST_RECORDING;
    	soundsList.setTitle("recording");
    }
    
    private void setPlaying(){
    	isRecording = ST_PLAYING;
    	soundsList.setTitle("playing");
    }
    
    private void setIdle(){
    	isRecording = ST_IDLE;
    	soundsList.setTitle("idle");
    }
	
    private void listen(){
    	new Thread(){
    		public void run(){
    		    while (true){
    		    	for ( int index = 0; index < messages.size(); index++){
        				while (isRecording != ST_IDLE){
        					try {Thread.sleep(100);} catch (InterruptedException e){};
        				}
        				setPlaying();
        		    	TMsg lMsg = (TMsg)messages.elementAt(index);
            			ByteArrayInputStream in = new ByteArrayInputStream( lMsg.msg );
        		  		try{
        		  			player = javax.microedition.media.Manager.createPlayer(in,"audio/amr");
        		  		} catch(Exception e){        	
        		  			informError("createPlayer "+e.toString());
        		      	} 
        		  			
        		  		if (player != null){
        		  			try{
        		      			player.realize();	
        		      			player.prefetch();
        		      			player.start();
        		      			Thread.sleep(lMsg.duration); 
        		      			player.stop();
        		      			player.close();
                		  		messages.removeElementAt(index);
                		  		onMessages();
        		      		}catch(Exception e){        	
        		      			informError("player "+e.toString());
        		          	}
        		  		}
        		  		try {in.close();} catch (IOException e){};
        		  		setIdle();
        			}
    		    	try {Thread.sleep(50);} catch (InterruptedException e){};
    		    }
    			
    		}
    	}.start();
  		
	}
    
    long calculateHash(byte[] byteArray){
		long hash = 0;
		for (int i = 0; i < byteArray.length; i++){
			hash+=(int)byteArray[i];
		}
		return hash;
    }
    
    private void recordAndSend(){
    	new Thread(){
    		public void run(){
    			long duration = 0;
    			setRecording();
    			ByteArrayOutputStream output = new ByteArrayOutputStream();
    	    	try{
    				player = javax.microedition.media.Manager.createPlayer("capture://audio?encoding=amr");
    				player.realize();
    				rc = (RecordControl)player.getControl("RecordControl");
    				rc.setRecordStream(output);
    		        rc.startRecord();
    		        player.start();
    		        
    		        duration = System.currentTimeMillis();
    		        while (isRecording == ST_RECORDING){
    		        	Thread.sleep(100);
    		        }
    		        duration = System.currentTimeMillis() - duration + 500;
    		
    		        rc.stopRecord();
    		        player.stop();
    		        rc.commit();
    		    	player.close();
    		    	
    		    	
    	    	}catch(Exception e){Console.trace("at record " + e.toString());}
    		    
    		     byte[] recordedSoundArray = output.toByteArray();
    		     try{output.close();}catch(IOException e){};

    		     
    			if (recordedSoundArray.length != 0){
    				sendSound(recordedSoundArray,duration,BTConnection.conn);
    			}
				 
    		}
    	}.start();
    	
     	
    }
    
    private void sendSound(final byte[] sound, final long duration, L2CAPConnection conn){
    	new Thread(){
    		public void run(){
		    	byte[] btOutput;
			     ByteArrayOutputStream boStream = new ByteArrayOutputStream();
			     oStream = new DataOutputStream(boStream);
			     if (oStream != null) {
			    	 try{
				    	 oStream.writeInt(P_MESSAGE);
				    	 oStream.writeInt(sound.length);
				    	 oStream.writeInt((int)duration);
				    	 for (int i = 0; i < sound.length; i++){
				    		 oStream.writeByte(sound[i]);
				    	 }
				    	 btOutput = boStream.toByteArray();
				    	 oStream.close();
				    	 boStream.close();
							
				    	 Misc.sendArray(btOutput,BTConnection.conn);
			    	 }catch (Exception e){
			    		 Console.trace("on data send " + e.toString());
			    		 onDisconnect();
			    	 };
			     }
    		}
    	}.start();
    }
	
    public void commandAction(Command c, Displayable d) {
    	if (c == cmdRec){
    		if (recOrSend){
    			if (isRecording == ST_IDLE){
    				recordAndSend();
        			soundsList.setTitle("recording");
        			recOrSend = false;
    			}
    		} else {
    			setIdle();
    			recOrSend = true;
    		}
    	} else if (c == cmdConsole){
    		Console.show(display);
    	}
    }

}
